@@ -1,5 +1,84 @@
-//! DHCPv4 Message type
-use std::net::Ipv4Addr;
+//! # DHCPv4
+//!
+//! This module provides types and utility functions for encoding/decoding a DHCPv4 message.
+//!
+//! ## Example - constructing messages
+//!
+//! ```rust
+//! # fn main() -> Result<(), Box<dyn std::error::Error>> {
+//! use dhcproto::{v4, Encodable, Encoder};
+//! // arbitrary hardware addr
+//! let chaddr = vec![
+//!     29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44,
+//! ];
+//! // construct a new Message
+//! let mut msg = v4::Message::default();
+//! msg.set_flags(v4::Flags::default().set_broadcast()) // set broadcast to true
+//!     .set_chaddr(&chaddr) // set chaddr
+//!     .opts_mut()
+//!     .insert(v4::DhcpOption::MessageType(v4::MessageType::Discover)); // set msg type
+//!
+//! // set some more options
+//! msg.opts_mut()
+//!     .insert(v4::DhcpOption::ParameterRequestList(vec![
+//!         v4::OptionCode::SubnetMask,
+//!         v4::OptionCode::Router,
+//!         v4::OptionCode::DomainNameServer,
+//!         v4::OptionCode::DomainName,
+//!     ]));
+//! msg.opts_mut()
+//!     .insert(v4::DhcpOption::ClientIdentifier(chaddr));
+//!
+//! // now encode to bytes
+//! let mut buf = Vec::new();
+//! let mut e = Encoder::new(&mut buf);
+//! msg.encode(&mut e)?;
+//!
+//! // buf now has the contents of the encoded DHCP message
+//! # Ok(()) }
+//! ```
+//!
+//! ## Example - decoding messages
+//!
+//! ```rust
+//! #  fn bootreq() -> Vec<u8> {
+//! #        vec![
+//! #            1u8, // op
+//! #            2,   // htype
+//! #            3,   // hlen
+//! #            4,   // ops
+//! #            5, 6, 7, 8, // xid
+//! #            9, 10, // secs
+//! #            11, 12, // flags
+//! #            13, 14, 15, 16, // ciaddr
+//! #            17, 18, 19, 20, // yiaddr
+//! #            21, 22, 23, 24, // siaddr
+//! #            25, 26, 27, 28, // giaddr
+//! #            29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, // chaddr
+//! #            45, 46, 47, 48, 49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65, 66,
+//! #            67, 68, 69, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 86, 87, 88,
+//! #            89, 90, 91, 92, 93, 94, 95, 96, 97, 98, 99, 100, 101, 102, 103, 104, 105, 106, 107,
+//! #            0, // sname: "-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijk",
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 110, 111, 112, 113, 114, 115, 116, 117, 118, 119, 120, 121, 122, 123, 124, 125,
+//! #            109, 0, 0, 0, 0, 0, 0, 0,
+//! #            0, // file: "mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}mnopqrstuvwxyz{|}m",
+//! #            99, 130, 83, 99, // magic cookie
+//! #        ]
+//! #    }
+//! # fn main() -> Result<(), Box<dyn std::error::Error>> {
+//! use dhcproto::{v4::Message, Decoder, Decodable};
+//! let offer = bootreq();
+//! let msg = Message::decode(&mut Decoder::new(&offer))?;
+//! # Ok(()) }
+//! ```
+//!
+use std::{net::Ipv4Addr, str::Utf8Error};
 
 mod flags;
 mod htype;
@@ -78,9 +157,9 @@ pub struct Message {
     /// Client hardware address
     chaddr: [u8; 16],
     /// Server hostname
-    sname: Option<String>,
+    sname: Option<Vec<u8>>,
     // File name
-    file: Option<String>,
+    fname: Option<Vec<u8>>,
     magic: [u8; 4],
     opts: DhcpOptions,
 }
@@ -101,7 +180,7 @@ impl Default for Message {
             giaddr: Ipv4Addr::UNSPECIFIED,
             chaddr: [0; 16],
             sname: None,
-            file: None,
+            fname: None,
             magic: MAGIC,
             opts: DhcpOptions::default(),
         }
@@ -300,30 +379,54 @@ impl Message {
         self.xid = xid;
         self
     }
-    /// Get a reference to the message's file.
-    pub fn file(&self) -> Option<&String> {
-        self.file.as_ref()
+    /// Get a reference to the message's fname. No particular encoding is enforced.
+    pub fn fname(&self) -> Option<&[u8]> {
+        self.fname.as_deref()
     }
-    /// Set the message's file.
+    /// Get a reference to the message's fname, UTF-8 encoded
+    pub fn fname_str(&self) -> Option<Result<&str, Utf8Error>> {
+        self.fname().map(std::str::from_utf8)
+    }
+    /// Set the message's fname using a UTF-8 string
     /// # Panic
     /// panics if file is greater than 128 bytes long
-    pub fn set_file<S: Into<String>>(&mut self, file: S) -> &mut Self {
-        let file = file.into();
+    pub fn set_fname_str<S: AsRef<str>>(&mut self, file: S) -> &mut Self {
+        let file = file.as_ref().as_bytes();
         assert!(file.len() <= 128);
-        self.file = Some(file);
+        self.fname = Some(file.to_vec());
         self
     }
-    /// Get a reference to the message's sname.
-    pub fn sname(&self) -> Option<&String> {
-        self.sname.as_ref()
+    /// Set the message's fname. No particular encoding is enforced.
+    /// # Panic
+    /// panics if file is greater than 128 bytes long
+    pub fn set_fname(&mut self, file: &[u8]) -> &mut Self {
+        assert!(file.len() <= 128);
+        self.fname = Some(file.to_vec());
+        self
     }
-    /// Set the message's sname.
+    /// Get a reference to the message's sname. No particular encoding is enforced.
+    pub fn sname(&self) -> Option<&[u8]> {
+        self.sname.as_deref()
+    }
+    /// Get a reference to the message's sname as a UTF-8 encoded string.
+    pub fn sname_str(&self) -> Option<Result<&str, Utf8Error>> {
+        self.sname().map(std::str::from_utf8)
+    }
+    /// Set the message's sname. No particular encoding is enforced.
     /// # Panic
     /// panics will if sname is greater than 64 bytes long
-    pub fn set_sname<S: Into<String>>(&mut self, sname: S) -> &mut Self {
-        let sname = sname.into();
+    pub fn set_sname(&mut self, sname: &[u8]) -> &mut Self {
         assert!(sname.len() <= 64);
-        self.sname = Some(sname);
+        self.sname = Some(sname.to_vec());
+        self
+    }
+    /// Set the message's sname using a UTF-8 string
+    /// # Panic
+    /// panics will if sname is greater than 64 bytes long
+    pub fn set_sname_str<S: AsRef<str>>(&mut self, sname: S) -> &mut Self {
+        let sname = sname.as_ref().as_bytes();
+        assert!(sname.len() <= 64);
+        self.sname = Some(sname.to_vec());
         self
     }
     /// Get a reference to the message's opts.
@@ -358,8 +461,8 @@ impl Decodable for Message {
             siaddr: decoder.read_u32()?.into(),
             giaddr: decoder.read_u32()?.into(),
             chaddr: decoder.read::<16>()?,
-            sname: decoder.read_const_string::<64>()?,
-            file: decoder.read_const_string::<128>()?,
+            sname: decoder.read_nul_bytes::<64>()?,
+            fname: decoder.read_nul_bytes::<128>()?,
             // TODO: check magic bytes against expected?
             magic: decoder.read::<4>()?,
             opts: DhcpOptions::decode(decoder)?,
@@ -381,8 +484,8 @@ impl Encodable for Message {
         e.write_u32(self.siaddr.into())?;
         e.write_u32(self.giaddr.into())?;
         e.write_slice(&self.chaddr[..])?;
-        e.write_fill_string(&self.sname, 64)?;
-        e.write_fill_string(&self.file, 128)?;
+        e.write_fill(&self.sname, 64)?;
+        e.write_fill(&self.fname, 128)?;
 
         e.write(self.magic)?;
         self.opts.encode(e)?;
